import 'package:data_connection_checker/data_connection_checker.dart';

import '../../m4e_sdk_initialize.dart';
import '../../core/core.dart';
import '../../m4e_forms/m4e_forms.dart';
import '../api/apis.dart';

/// Exposes SDK User Api methods
///
/// Accessing the instantiated `M4eGroupService` methods
/// is done by:
///
/// ```
/// final _groupService = M4eGroupService();
///
/// final _groupId = await _groupService.createGroup(M4eGroupForm m4eGroupForm);
/// ```
///
class M4eGroupService {
  final GroupApi _groupApi;
  final AuthApi _authApi;
  final DataConnectionChecker _connectionChecker;

  M4eGroupService({
    GroupApi groupApi,
    AuthApi authApi,
    DataConnectionChecker connectionChecker,
  })  : _groupApi = groupApi ?? m4eSDK<GroupApi>(),
        _authApi = authApi ?? m4eSDK<AuthApi>(),
        _connectionChecker =
            connectionChecker ?? m4eSDK<DataConnectionChecker>();

  /// Creates `m4e` group with provided [M4eGroupForm] details
  ///
  /// Returns [M4eUniqueId] of created group on success
  ///
  /// Throws [M4eServerException] if request fails.
  ///
  /// Throws `[M4eServerException]` with `NO_INTERNET_CONNECTION` error
  /// code when device is not connected to the internet
  ///
  Future<M4eUniqueId> createGroup(M4eGroupForm m4eGroupForm) async {
    if ((await _connectionChecker.hasConnection) ?? false) {
      try {
        await _authApi.getAuthTokenFromCache();

        final _associationId = await _groupApi.createGroup(m4eGroupForm);

        return _associationId;
      } catch (_) {
        rethrow;
      }
    }

    throw M4eExceptionMessages.kNoInternetConnectionException;
  }

  /// Adds [GroupEntityRole]s an existing`m4e` group
  ///
  /// Returns [List<M4eUniqueId>] of added entity-roles on success
  ///
  /// Throws [M4eServerException] if request fails.
  ///
  /// Throws `[M4eServerException]` with `NO_INTERNET_CONNECTION` error
  /// code when device is not connected to the internet
  ///
  Future<List<M4eUniqueId>> addGroupEntityRoles(
      M4eGroupEntityRoleForm groupEntityRoleForm) async {
    if ((await _connectionChecker.hasConnection) ?? false) {
      try {
        await _authApi.getAuthTokenFromCache();

        final _entityRoleIds =
            await _groupApi.addGroupEntityRoles(groupEntityRoleForm);

        return _entityRoleIds;
      } catch (_) {
        rethrow;
      }
    }

    throw M4eExceptionMessages.kNoInternetConnectionException;
  }
}
